package projet_these.et;

import projet_these.rn.*;
import java.util.*;
import java.io.*;
import java.lang.*;




/**
 * Implementation of a pair of chromosomes
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 01/03/02
 */
public class ChromosomesPair implements Serializable
{

    /** 
     * Numbers of insertions of the 2 pairs
     */
    protected int nb_ins;

    /**
     * Numbers of insertions of the a pair
     */
    protected int nb_ins_a;
    
        /**
     * Default recombination rate between the chromosome tips
     */
    public final static float DEFAULT_REC_RATE = 0.9f;

    /**
     * Current recombination rate between the chromosome tips
     */
    private static float rec_rate = DEFAULT_REC_RATE;

    /**
     * Chromosome a of the pair
     */
    protected Hashtable inser_a;

    /**
     * Chromosome b of the pair
     */
    protected Hashtable inser_b;


    /**
     * Empty Contructor of ChromosomesPair
     */
    public ChromosomesPair()
    {
    }


    /**
     * Constructor of initialization of chromosomes
     * @param n Total number of copies of the pair
     */
    public ChromosomesPair(int n)
    {
        // Chromosoms initiation
        inser_a = new Hashtable();
        inser_b = new Hashtable();

        // Insertions number initiation for a pair
        nb_ins = n;

        //Insterts positioning process & initiation of inserts
        nb_ins_a = transpoIns(nb_ins, false);

    }


    /**
     * Procedure setting a pair of chromosomes after gametic fusion
     * @param i_a The a Chromosome
     * @param i_b The b Chromosome
     * @return CromosomesPair the setted pair
     */
    protected ChromosomesPair setChromo(Hashtable i_a, Hashtable i_b)
    {

        // Chromosomes in one pair
        inser_a = cloneChromo(i_a);
        inser_b = cloneChromo(i_b);

        // Insertions number initiation for a pair
        nb_ins_a = 0;

        // Puts chromosom a keys in a table
        Object[] keys = inser_a.keySet().toArray();

        // Insertion number determination on inser_a key after key
        for (int i = 0; i < keys.length; i++)
            nb_ins_a += ((LinkedList) inser_a.get((Byte) keys[i])).size();

        // Denombrement global des insertions sur la paire
        nb_ins = nb_ins_a;
        keys = inser_b.keySet().toArray();
        for (int i = 0; i < keys.length; i++)
            nb_ins += ((LinkedList) inser_b.get((Byte) keys[i])).size();

        return this;
    }


    /**
       * Function of segregation of the homologous chromosomes in meiose
       * @return the chromosome constitutive of the gamete
       */
    protected Hashtable segregation()
    {
        // Schema of Bernoulli
        if ((int) bnldev.binom(0.5f, 1) == 1)
            return recombinaison(inser_a, inser_b);/*inser_a*/
        else
            return recombinaison(inser_b, inser_a);/*inser_b*/
    }


    /**
       * Function of recombainaison during meiosis
       * @param chromo_gam The gametic Chromosome
       * @param chromo_homo The homologous Chromosome
       * @return The gametic Chromosome after recombinaison
       */
    protected Hashtable recombinaison(Hashtable chromo_gam,
            Hashtable chromo_homo)
    {

        // Taking of recombinations number
        int n_recom = (int) poidev.poisson(ChromosomesPair.rec_rate);

        // Clons of the chromosoms pair
        Hashtable chromo_meio = cloneChromo(chromo_gam);
        Hashtable clone_chrho = cloneChromo(chromo_homo);
        if (n_recom == 0)
            return chromo_meio;

        // Initiation of recombination positions array
        List positions = new ArrayList();

        /* Initiation of inserts array of :
        	- gametic chromosom
        	- homologous chromosom*/
        List keys_gam = new ArrayList(chromo_meio.keySet());
        List keys_homo = new ArrayList(clone_chrho.keySet());

        // Racombinations positionning
        Byte position_i;
        for (int i = 0; i < n_recom; i++)
        {
            do
            {
                position_i =
                        new Byte((byte)(1 + (int)(120.0 * Math.random())));
            }
            while (positions.contains(position_i))
                ;
            positions.add(position_i);
        }
        positions.add(new Byte((byte) 121));

        // Arrays shorting
        Collections.sort(positions);
        Collections.sort(keys_gam);
        Collections.sort(keys_homo);

        /* Pour chaque position recombinante d'ordre impaire
        on recupere les clefs echangees (i.e. entre la position
        recombinante courante et la precedente) entre le chromosome
        gametique et son homologue*/
        int j = 0, k = 0;
        LinkedList inser_key = new LinkedList();
        for (int i = 1; i < positions.size(); i += 2)
        {

            // Exchange from chromo_gam to chromo_homo
            for (; j < keys_gam.size() &&
                    ((Byte) keys_gam.get(j)).intValue() <
                    ((Byte) positions.get(i)).intValue(); j++)
            {
                if (((Byte) keys_gam.get(j)).intValue() >
                        ((Byte) positions.get(i - 1)).intValue())
                {
                    chromo_meio.remove(keys_gam.get(j));
                }
            }

            // Excange from chromo_homo to chromo_gam
            for (; k < keys_homo.size() &&
                    ((Byte) keys_homo.get(k)).intValue() <
                    ((Byte) positions.get(i)).intValue(); k++)
            {
                if (((Byte) keys_homo.get(k)).intValue() >
                        ((Byte) positions.get(i - 1)).intValue())
                {
                    try
                    {
                        inser_key = ((LinkedList) clone_chrho.get(
                                keys_homo.get(k)));
                    }
                    catch (NullPointerException e)
                    {}
                    chromo_meio.put(keys_homo.get(k), inser_key);
                }
            }
        }

        return chromo_meio;
    }


    /**
     * Function of transposition during meiosis
     * @param new_ins The new copies number on the chromosomes pair
     * @param invasive Boolean
     * @return new_a    The new copies number on the inser_a
     */
    protected int transpoIns(int new_ins, boolean invasive)
    {

        // New insertion number on chromosom a
        int new_a = (int) bnldev.binom(0.5f, new_ins);

        // Each new gametic chromosom insertion positionning on one of 120 strip
        Byte key;
        for (int i = 0; i < new_a; i++)
        {
            key = new Byte((byte)(1 + (int)(120.0 * Math.random())));
            if (inser_a.containsKey(key))
                ((LinkedList) inser_a.get(key)).add(
                        new transElement(invasive));
            else
            {
                inser_a.put(key, new LinkedList());
                ((LinkedList) inser_a.get(key)).add(
                        new transElement(invasive));
            }
            /* Remarque 1 : le tirage uniforme d'un nombre x element de
              	 [alpha,beta] se fait par :
              	 alpha+(beta-alpha+1)*Math.random()
              	 Remarque 2 : plusieurs insertions differentes
              	 peuvent etre sur la meme bande */
        }

        // Each new a chromosom insertion positionning on one of 120 strip
        for (int i = 0; i < (new_ins - new_a); i++)
        {
            key = new Byte((byte)(1 + (int)(120.0 * Math.random())));
            if (inser_b.containsKey(key))
                ((LinkedList) inser_b.get(key)).add(
                        new transElement(invasive));
            else
            {
                inser_b.put(key, new LinkedList());
                ((LinkedList) inser_b.get(key)).add(
                        new transElement(invasive));
            }
            /* Remarque 1 : le tirage uniforme d'un nombre x element de
              	 [alpha,beta] se fait par :
              	 alpha+(beta-alpha+1)*Math.random()
              	 Remarque 2 : plusieurs insertions differentes
              	 peuvent etre sur la meme bande */
        }

        return new_a;
    }


    /**
       * Function of excision during meiosis
       * @param nb_exc Number of excisions on the chromosomes pair
       */
    protected void excisIns(int nb_exc)
    {

        // Excision number on a & b chromosoms
        int past_a =
            (int) bnldev.binom(this.getCopNumber_a() /
            ((float) this.getCopNumber()), nb_exc);
        int past_b = nb_exc - past_a;
        if (past_a >= this.getCopNumber_a())
        {
            past_a = this.getCopNumber_a();
            past_b = nb_exc - past_a;
        }
        else if (past_b >= this.getCopNumber() - this.getCopNumber_a())
        {
            past_b = this.getCopNumber() - this.getCopNumber_a();
            past_a = nb_exc - past_b;
        }

        /* Initiation of inserts tables of:
        	- chromosom a
        	- chromosom b*/
        List keys_a = new ArrayList(inser_a.keySet());
        List keys_b = new ArrayList(inser_b.keySet());

        // Past gametic chromosom's insertions positionning on one of 120 strips
        Byte key;

        // Insertion site
        LinkedList insertion;

        // Excisions index on each chromosoms
        int index_a, index_b;

        // Past a chromosom's insertions positionning on one of 120 strips
        for (int i = 0; i < past_a; i++)
        {
            index_a = (int)(((double) keys_a.size()) * Math.random());
            key = (Byte) keys_a.get(index_a);
            if (((LinkedList) inser_a.get(key)).size() > 1)
            {
                insertion = ((LinkedList) inser_a.get(key));
                insertion.remove((int)(insertion.size() * Math.random()));
            }
            else
            {
                inser_a.remove(key);
                keys_a.remove(index_a);
            }
        }
        /* Remarque : le tirage uniforme d'un nombre x element de
              	 [alpha,beta] se fait par :
              	 alpha+(beta-alpha+1)*Math.random()*/

        // Past b chromosom's insertions positionning on one of 120 strips
        for (int i = 0; i < past_b; i++)
        {
            index_b = (int)(((double) keys_b.size()) * Math.random());
            key = (Byte) keys_b.get(index_b);
            if (((LinkedList) inser_b.get(key)).size() > 1)
            {
                insertion = ((LinkedList) inser_b.get(key));
                insertion.remove((int)(insertion.size() * Math.random()));
            }
            else
            {
                inser_b.remove(key);
                keys_b.remove(index_b);
            }
        }
        /* Remarque : le tirage uniforme d'un nombre x element de
              	 [alpha,beta] se fait par :
              	 alpha+(beta-alpha+1)*Math.random()*/
    }


    /**
       * Function cloning a chromosome
       * @param chromo  The chromosome to clone
       * @return        The cloned chromosome
       */
    protected Hashtable cloneChromo(Hashtable chromo)
    {
        // The clone
        Hashtable clone = new Hashtable();

        // Keys of chromosom in an array
        Object[] keys = chromo.keySet().toArray();

        // An insertion site
        Object[] ins_site;

        // Insertion number determination on inser_a key after key
        for (int i = 0; i < keys.length; i++)
        {
            clone.put(
                    new Byte(((Byte)keys[i]).byteValue()),
                    new LinkedList());
            ins_site = ((LinkedList) chromo.get((Byte) keys[i])).
                toArray();
            for (int j = 0; j < ins_site.length; j++)
                ((LinkedList) clone.get((Byte) keys[i])).add(
                    ((geneAllele) ins_site[j]).clone());
        }

        return clone;
    }


    /**
     * Procedure setting recombination rate between the chromosome tips
     * @param rate The recombination rate between the chromosome tips
     */
    public static void setRecRate(float rate)
    {
        ChromosomesPair.rec_rate = rate;
    }
    

    /**
     * Functin returning the total copy number
     * @return  Number of elements
     */
    protected int getCopNumber()
    {
        return this.nb_ins;
    }


    /**
     * Functin returning the copy number on the "a" chromosome
     * @return  Number of elements on the "a"
     */
    protected int getCopNumber_a()
    {
        return this.nb_ins_a;
    }


    /**
     * Function returning recombination rate between the chromosome tips
     * @return              rec_rate value
     */
    static public float getRecRate()
    {
        return ChromosomesPair.rec_rate;
    }
}


