package projet_these.et;

import projet_these.rn.*;
import java.io.*;
import java.util.*;




/**
 * Implementation of a genome
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 01/03/02
 */
public class Individu implements Serializable
{
    
    /**
     * Number of copies of the individual
     */
    protected int nb_copies;
    
    /**
     * State of transposition
     */
    protected boolean tran_exc;
    
    /**
     * The three pairs of chromosomes of the genome
     */
    protected ChromosomesPair[] XX;
    
    /**
     * Types of the chromosomes pairs
     */
    protected static final ChromosomesPair chromo_pairs_class
            = new ChromosomesPair();
    
    
    /**
     * Empty Contructor of an individual
     */
    public Individu()
    {
    }
    
    
    /**
     * Contructor of initialization of the individuals
     * @param n Total number of copies of the genome
     */
    public Individu(int n)
    {
        // Inserts number on each pairs
        int[] ni = new int[3];
        
        // Genome copies number initiation
        nb_copies = n;
        
        //  Running of positionning process
        transposition(nb_copies, ni, 1f);
        
        // Chromosomes pair initiation
        XX = new ChromosomesPair[3];
        for (int i = 0; i < 3; i++)
            XX[i] = new ChromosomesPair(ni[i]);
        
        // Transposition state initiation
        tran_exc = false;
    }
    
    
    /**
     * Procedure setting a genome
     * @param X First pair of chromosomes
     * @param Y Second pair of chromosomes
     * @param Z Third pair of chromosomes
     * @return Individu the genome setted
     */
    protected Individu setChromoPairs(ChromosomesPair X, ChromosomesPair Y, ChromosomesPair Z)
    {
        // 3 pairs of chromosoms in XX
        XX = new ChromosomesPair[3];
        XX[0] = X;
        XX[1] = Y;
        XX[2] = Z;
        
        // Genomic copies number initiation
        nb_copies = XX[0].nb_ins + XX[1].nb_ins + XX[2].nb_ins;
        
        // Transposition state initiation
        tran_exc = false;
        
        return this;
    }
    
    
    /**
     * Procedure of transpositions on the differents pairs of chromosomes
     * @param nb_cop  Number of genomic insertions
     * @param nb_x    Numbers of insertions on each chromosomes paires
     * @param rate    Transposition rate of this insertions
     */
    protected void transposition(int nb_cop, int[] nb_x, float rate)
    {
        
        //fi positioning on one of the 3 pairs parameter declaration
        // and insertion number on each pairs declaration
        int fi;
        Arrays.fill(nb_x, 0);
        
        // New inserts  number
        int new_inser = (nb_cop > 0) ?
            (int) bnldev.binom(rate, nb_cop) : 0;
        if (new_inser == 0) return;
        
        // Insertion number determination on each pairs
        for (int i = 0; i < new_inser; i++)
        {
            
            fi = 1 + (int)(3.0 * Math.random());
            /* Remarque 1 : le tirage uniforme d'un nombre x element de
              [alpha,beta] se fait par :
              alpha+(beta-alpha+1)*Math.random()*/
            
            switch (fi)
            {
                
                // Insertion is on the first pair
                case 1 :
                    nb_x[0] += 1;
                    break;
                    
                // Insertion on the second
                case 2 :
                    nb_x[1] += 1;
                    break;
                    
                // Insertion on the third
                case 3 :
                    nb_x[2] += 1;
                    break;
            }
        }
        
        return;
    }
    
    
    /**
     * Procedure of positioning insertions on the pairs of chromosomes
     * @param nb_x New copies number on each pairs
     * @param invasive boolean
     */
    protected void insCop(int[] nb_x, boolean invasive)
    {
        // Running of insertions-excisions positioning process
        for (int i = 0; i < 3; i++)
        {
            if (nb_x[i] > 0)
                XX[i].transpoIns(nb_x[i], invasive);
        }
    }
    
    
    /**
     * Procedure of positioning excisions on the pairs of chromosomes
     */
    protected void delCop()
    {
        // Number of old inserts on each pairs
        int[] nb_delcop = new int[3];
        
        // New deletions must be positioned for each pairs on one of the both chromosomes
        for (int i = 0; i < 3; i++)
        {
            nb_delcop[i] = (XX[i].getCopNumber() > 0) ?
                (int) bnldev.binom(transElement.getExciRate(),
                    XX[i].getCopNumber()) : 0;
            if (nb_delcop[i] > 0)
                XX[i].excisIns(nb_delcop[i]);
        }
    }
    
    
    /**
     * Function of segregation of the homologous pairs of chromosomes in meiose
     * @return the table of the separate chromosomes of their counterpart,
     * i.e. constitutive of the gamete
     */
    protected Hashtable[] gametes()
    {
        
        // Haplotype initation
        Hashtable[] haplo = new Hashtable[3];
        
        // Transposition/Excision of all germinales cells in the same way
        if (! tran_exc)
        {
            int[] nb_x = new int[3];
            transposition(this.nb_copies, nb_x, transElement.getNormTransRate());
            insCop(nb_x, false);
            delCop();
            
            // Transposition state changing
            tran_exc = true;
        }
        
        // Homologues separation during meiotic step
        for (int i = 0; i < 3; i++)
            haplo[i] = XX[i].segregation();
        
        return haplo;
    }
    
    
    /**
     * Functin returning selected entity
     * @return  Number of selected elements
     */
    protected int getEntite_sel()
    {
        return this.nb_copies;
    }
}


