package projet_these.et;

import java.util.*;




/**
 * Implementation of a individu under the local burst hypothesis
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 20/12/02
 */
public class IndividuLoc extends Individu
{
    
    /**
     * The population of reception
     */
    private PopulationLocalBurst birth_pop;
    
    /**
     * Types of the chromosomes pairs
     */
    protected static final ChromosomesPair chromo_pairs_class
            = new ChromosomesPair();
    
    /**
     * Empty Contructor of an individual
     */
    public IndividuLoc()
    {
    }
    
    
    /**
     * Contructor of initialization of the individuals
     * @param pop PopulationLocalBurst
     * @param n Total number of copies of the genome
     */
    public IndividuLoc(int n, PopulationLocalBurst pop)
    {
        super(n);
        setBirthPop(pop);
    }
    
    
    /**
     * Procedure setting the birth place of the individu
     * @param pop   The birth place
     */
    protected void setBirthPop(PopulationLocalBurst pop)
    {
        this.birth_pop = pop;
    }
    
    
    /**
     * Function of segregation of the homologous pairs of chromosomes in meiose
     * @return the table of the separate chromosomes of their counterpart,
     * i.e. constitutive of the gamete
     */
    protected Hashtable[] gametes()
    {
        
        // Initialisation of the haplotype
        Hashtable[] haplo = new Hashtable[3];
        
        // Transposition/Excision of all germinales cells in the same way
        if (! tran_exc)
        {
            int[] nb_x = new int[3];
            transposition(this.nb_copies, nb_x, birth_pop.getTransRate());
            insCop(nb_x, false);
            delCop();
            
            // Transposition state changing
            tran_exc = true;
        }
        
        // Homologues separation during meiotic step
        for (int i = 0; i < 3; i++)
            haplo[i] = XX[i].segregation();
        
        return haplo;
    }
}