package projet_these.et;

import projet_these.rn.*;




/**
 * Implementation of a population under the local burst hypothesis
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 20/12/02
 */
public class PopulationLocalBurst extends Population
{
    
    // 
    /**
     * Local Transposition rate
     */
    private float tx_transpo;
    
    
    /**
     * Contructor of initialization of the population
     * @param   t           Population size
     * @param   n           Average of the number of copies at initial time
     * @param   invasive    The type of population
     * @param   g           Number of simulated generations
     */
    public PopulationLocalBurst(int t, float n, boolean invasive, int g)
    {
        super(t, g);
        
        // Initiation of individuals of the population
        for (int i = 0; i < t; i++)
            population.add(new IndividuLoc((int)(poidev.poisson(n)), this));
        
        // Copies profile initiation
        setFrequencies();
        
        // Real initial mean initiation
        moyenne(0);
        
        // Setting the local rate of transposition
        this.tx_transpo = invasive ? transElement.getInvaTransRate() :
            transElement.getNormTransRate();
        
        // Set the birth location
        this.setLocal();
    }
    
    
    /**
     * Procedure setting individual rate of transposition
     * relatively to the population of birth
     */
    protected void setLocal()
    {
        
        for (int i = 0; i < population.size(); i++)
        {
            ((IndividuLoc) population.elementAt(i)).setBirthPop(this);
        }
    }
    
    
    /**
     * Function returning population rate of transposition
     * relatively to the population of birth
     * @return    The population rate of transposition
     */
    protected float getTransRate()
    {
        return this.tx_transpo;
    }
    
    
    /**
     * Procedure of reproduction of individuals monoecious diploids
     * excluding self fertilization
     */
    protected void reproduction()
    {
        super.reproduction();
        setLocal();
    }
    
}