/*************************************************************************
*         Copyright notice for JFreeChart and JCommon libraries          *
*                                                                        *
*The present application links to JFreeChart and JCommon libraries as    *
*external jar files that are under the GNU Lesser General Public License.*
*You can find a copy of that license at the root folder containing       *
*the main class of TESD. More informations such as complete source codes *
*to modify the libraries are available at this web site: www.jfree.org.  *
*************************************************************************/

package projet_these.ig;

import java.awt.*;
import org.jfree.chart.*;
import org.jfree.chart.plot.*;
import org.jfree.data.xy.*;



/**
 * A Graphiques inherited class used to plot the .var and .mea files on a curve.
 * @author Yann Letrillard
 * @version 1.0
 */
public class Curves extends Graphiques {
    
    /**
     * The dataset for curve plotting. Filled by setCurvesDataset() and used by 
     * chartfactory methods
     */
    private DefaultTableXYDataset dataset;
    //[GD]private XYSeriesCollection dataset;
    
    /**
     * The JFreeChart XYLineChart ploting the data
     */
    private JFreeChart curve;
    
    
    /**
     * Construct a new Curves
     * @param output_file_name The file containing data (from a simulation) to plot mean or variances
     * @param super_frame The main frame used by the default Graphics contructor
     */
    public Curves(MainFrame super_frame, String output_file_name) {
        
        super(super_frame, "Curves", true, output_file_name);
        
        //If the choice of the file was successful, Curves is fully constructed
        if (this.dispose == false){
            
            this.setCurvesDataset();
            
            this.setCurve();
            
            this.setXAxis(curve);
            
            this.setChartPanel(curve);
        }
        
        
        
    }
    
    /**
     * Fills the DefaultTableXYDataset dataset of Curves from the output_table
     */
    private void setCurvesDataset() {
        
        //creates a new DefaultTableXYDataset
        this.dataset = new DefaultTableXYDataset();
        //[GD]this.dataset = new XYSeriesCollection();
        
        //i the number of the current line and j of the current column
        int i,j;
        
        for (j=1; j <= getTableWidth()-1; j++) {
            
            //Sets the name of the current jth population
            String population_name = "population" + j;
            
            //Creates and fills the Series fro the current jth population (sort from X value
            //and doesn't allow X replicants values
            //[GD]XYSeries the_j_population_series = new XYSeries(population_name,true,true);
            XYSeries the_j_population_series = new XYSeries(population_name,true,false);
            for (i=0; i < getTableHeight()-1; i++) {
                the_j_population_series.add(output_table[i][0], output_table[i][j]);
            }
            //The current Series is added to the dataset
            dataset.addSeries(the_j_population_series);
        }
    }
    
    /**
     * Instanciate the XYLineChart plot in curve
     */
    private void setCurve(){
        JFreeChart curve = ChartFactory.createXYLineChart(this.setChartTitle(),
                    "generations", "copy number means",
                    this.dataset, PlotOrientation.VERTICAL,
                    true, true, false);
        this.curve = curve;
    }
    
    /**
     * Create the chart title from the name of the input file
     * @return A String containing the title of the chart
     */
    private String setChartTitle(){
        String title = output_file_name.substring(output_file_name.lastIndexOf(System.getProperty("file.separator"))+1);
        return title;
    }
    
    /**
     * Set an appropriate autoscaling of the xAxis of the cruve chart
     * @param curve The JFreeChart to which the xAxis will be autoscalled
     */
    private void setXAxis(JFreeChart curve) {
        curve.getXYPlot().getDomainAxis().setUpperBound(getTableHeight()-2);
        curve.getXYPlot().getDomainAxis().setLowerBound(0);
        curve.getXYPlot().getDomainAxis().setFixedAutoRange(getTableHeight()-2);
    }
    
    /**
     * Create the ChartPanel that will welcome the chart
     * @param curve The JFreeChart curve that will be added to the chart panel
     */
    private void setChartPanel(JFreeChart curve) {
        ChartPanel chart_panel = new ChartPanel(curve);
        chart_panel.setPreferredSize(getSuperFrame().getSize());
        chart_panel.setMouseZoomable(true, false);
        chart_panel.getChart().getXYPlot().getDomainAxis().setAutoRange(true);
        chart_panel.getChart().getXYPlot().getRangeAxis().setAutoRange(true);
        
        this.getContentPane().add(chart_panel);
    }
    
    
}


