/*************************************************************************
*         Copyright notice for JFreeChart and JCommon libraries          *
*                                                                        *
*The present application links to JFreeChart and JCommon libraries as    *
*external jar files that are under the GNU Lesser General Public License.*
*You can find a copy of that license at the root folder containing       *
*the main class of TESD. More informations such as complete source codes *
*to modify the libraries are available at this web site: www.jfree.org.  *
*************************************************************************/

package projet_these.ig;

/*
 * NewJDialog.java
 *
 * Created on 9 mars 2005, 14:18
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.text.*;
import java.lang.*;
import java.io.*;
import java.util.*;

/**
 * The form used to set what is needed to Maps: a .mea or .var input file, the
 * coordinates of the populations and the interval of generations at which a chart
 * will be represented
 * @author amiv
 */
public class MapSettings extends JDialog {
    
    // Variables declaration
    /**
     * The "Browse" button to choose a mean or variance file
     */
    private JButton jButton1;
    /**
     * The "New entry" button that will instanciate a Coordinates object to let user
     * custom the coordinates
     */
    private JButton jButton2;
    /**
     * The "Browse" button to choose an existing .coo coordinates file
     */
    private JButton jButton3;
    /**
     * The "Ok" button to validates Map settings
     */
    private JButton jButton4;
    /**
     * The "Cancel" button to abort settings and return to the TESD main frame
     */
    private JButton jButton5;
    /**
     * Label "Mean or Variance input file:"
     */
    private JLabel jLabel1;
    /**
     * Label "Populations coordiantes"
     */
    private JLabel jLabel2;
    /**
     * Label "Interval of generations"
     */
    private JLabel jLabel3;
    /**
     * Label "for a number of"
     */
    private JLabel jLabel4;
    /**
     * Label "representations"
     */
    private JLabel jLabel5;
    /**
     * TextField showing the name of the .mea or .va input file
     */
    private JTextField jTextField1;
    /**
     * TextField showing the .coo file chosen or the automatic default setting
     */
    private JTextField jTextField2;
    /**
     * The text field capturing the interval set by the user or showing the
     * automatically calculated one
     */
    private JFormattedTextField jFormatedTextField3;
    /**
     * TextField showing the number a representations the interval gives
     */
    private JTextField jTextField4;
    
    /**
     * The map object that is setted
     */
    private Maps map;
    
    /**
     * Creates new graphical form MapSettings to set the map graphic.
     * @param file_chooser Boolean if true the user have already choosen a file and 
     * he can set other parameters
     * @param map The map object to be set. Used to get method and fields from it.
     * @param parent The main frame instance.
     * @param modal Boolean if true the JDialog MapSettings is modal
     */
    public MapSettings(Frame parent, Maps map, boolean modal, boolean file_chooser) {
        super(parent, modal);
        setMap(map);
        initComponents();
        if (file_chooser == false){
            jButton1.setEnabled(false);
            map.setOutputTable();
            map.autoSetCoordinates();
            jTextField1.setText(map.output_file_name);
            jTextField2.setFont(new Font("Dialog",Font.ITALIC, 12));
            jTextField2.setText("Automatic default settings");
            jFormatedTextField3.setEditable(true);
            map.autoSetInterval(-1);
            jFormatedTextField3.setValue(new Integer(map.getInterval()));
            jTextField4.setText(String.valueOf(map.getChartNumber()));
            jButton2.setEnabled(true);
            jButton3.setEnabled(true);
            jButton4.setEnabled(true);
        }
        if (this.isVisible() == false) map.disposeGraphic(true);
    }
    
    /**
     * This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        GridBagConstraints gridBagConstraints;
        
        jLabel1 = new JLabel();
        jTextField1 = new JTextField();
        jButton1 = new JButton();
        jButton1.setAction(new ActionManager("data"));
        jLabel2 = new JLabel();
        jTextField2 = new JTextField();
        jButton2 = new JButton();
        jButton2.setAction(new ActionManager("new coord"));
        jButton2.setEnabled(false);
        jButton3 = new JButton();
        jButton3.setAction(new ActionManager("coord"));
        jButton3.setEnabled(false);
        jLabel3 = new JLabel();
        //jTextField3 can only accept integer
        jFormatedTextField3 = new JFormattedTextField(NumberFormat.getIntegerInstance());
        jLabel4 = new JLabel();
        jTextField4 = new JTextField();
        jLabel5 = new JLabel();
        jButton4 = new JButton();
        jButton4.setAction(new ActionManager("ok"));
        jButton4.setEnabled(false);
        jButton5 = new JButton();
        jButton5.setAction(new ActionManager("cancel"));
        
        getContentPane().setLayout(new GridBagLayout());
        
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        setLocation(getParent().getLocation().x+10,getParent().getLocation().y+90);
        setTitle("Map settings");
        setResizable(false);
        
        Font default_font = new Font("Dialog", 0, 12);
        
        jLabel1.setFont(default_font);
        jLabel1.setText("Mean or Variance input file:");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.anchor = GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new Insets(7, 5, 2, 2);
        getContentPane().add(jLabel1, gridBagConstraints);
        
        jTextField1.setEditable(false);
        jTextField1.setPreferredSize(new Dimension(200, 19));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(7, 2, 2, 2);
        getContentPane().add(jTextField1, gridBagConstraints);
        
        jButton1.setFont(default_font);
        jButton1.setText("Browse...");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = GridBagConstraints.LINE_END;
        gridBagConstraints.insets = new Insets(7, 2, 2, 5);
        getContentPane().add(jButton1, gridBagConstraints);
        
        jLabel2.setFont(default_font);
        jLabel2.setText("Populations coordinates:");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.anchor = GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new Insets(2, 5, 2, 2);
        getContentPane().add(jLabel2, gridBagConstraints);
        
        jTextField2.setEditable(false);
        jTextField2.setPreferredSize(new Dimension(200, 19));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        getContentPane().add(jTextField2, gridBagConstraints);
        
        jButton2.setFont(default_font);
        jButton2.setText("New entry...");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = GridBagConstraints.EAST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        getContentPane().add(jButton2, gridBagConstraints);
        
        jButton3.setFont(default_font);
        jButton3.setText("Browse...");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = GridBagConstraints.LINE_END;
        gridBagConstraints.insets = new Insets(2, 2, 2, 5);
        getContentPane().add(jButton3, gridBagConstraints);
        
        jLabel3.setFont(default_font);
        jLabel3.setText("Interval of generations:");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.anchor = GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new Insets(2, 5, 2, 2);
        getContentPane().add(jLabel3, gridBagConstraints);
        
        jFormatedTextField3.setEditable(false);
        jFormatedTextField3.addPropertyChangeListener("value", new ActionManager(""));
        jFormatedTextField3.setPreferredSize(new Dimension(50, 19));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        getContentPane().add(jFormatedTextField3, gridBagConstraints);
        
        jLabel4.setFont(default_font);
        jLabel4.setText("for a number of ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        getContentPane().add(jLabel4, gridBagConstraints);
        
        jTextField4.setEditable(false);
        jTextField4.setPreferredSize(new Dimension(50, 19));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = GridBagConstraints.EAST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        getContentPane().add(jTextField4, gridBagConstraints);
        
        jLabel5.setFont(default_font);
        jLabel5.setText("representations");
        gridBagConstraints = new GridBagConstraints();
        //gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = new Insets(2, 2, 2, 2);
        getContentPane().add(jLabel5, gridBagConstraints);
        
        jButton4.setFont(default_font);
        jButton4.setText("OK");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = GridBagConstraints.LINE_END;
        gridBagConstraints.insets = new Insets(2, 0, 7, 2);
        getContentPane().add(jButton4, gridBagConstraints);
        
        jButton5.setFont(default_font);
        jButton5.setText("Cancel");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = GridBagConstraints.LINE_END;
        gridBagConstraints.insets = new Insets(2, 2, 7, 5);
        getContentPane().add(jButton5, gridBagConstraints);
        
        pack();
    }
    
    /**
     * Set the argument map to a field of MapSettings
     * @param map The Maps object to be set. Comes from the arguments of the constructor
     */
    private void setMap(Maps map) {
        this.map = map;
    }
    
    /**
     * The action manager dedicated to specify the action of the MapSettings buttons.
     */
    private class ActionManager extends AbstractAction implements PropertyChangeListener{
        
        
        /**
         * The constructor of action manager. Must contain a
         * putValue(String key, Object newValue) to associate the apropriate action to the
         * calling button.
         * @param action_name A String giving the name of the action to do by the actionParfomed method
         */
        private ActionManager(String action_name) {
            //A key name method to determine the action to do
            putValue(this.NAME, action_name);
        }
        
        /**
         * The method that does the differents actions associated to their name
         * @param e The action event
         */
        public void actionPerformed(ActionEvent e){
            
            //To choose an data fil
            if (getValue(this.NAME).equals("data")){
                map.setOutputFile();
                if (map.output_file_name == null)
                    return;
                else{
                    map.setOutputTable();
                    if (map.dispose == true)
                        return;
                    else {
                        map.setOutputTable();
                        map.autoSetCoordinates();
                        jTextField1.setText(map.output_file_name);
                        jTextField2.setFont(new Font("Dialog",Font.ITALIC, 12));
                        jTextField2.setText("Automatic default settings");
                        jFormatedTextField3.setEditable(true);
                        map.autoSetInterval(-1);
                        jFormatedTextField3.setValue(new Integer(map.getInterval()));
                        jTextField4.setText(String.valueOf(map.getChartNumber()));
                        jButton2.setEnabled(true);
                        jButton3.setEnabled(true);
                        jButton4.setEnabled(true);
                    }
                }
            }
            
            //To close everything
            if(getValue(this.NAME).equals("cancel")){
                map.disposeGraphic(true);
                dispose();
            }
            
            //To choose a coordinate file
            if(getValue(this.NAME).equals("coord")){
                File coord_file = getCoordinatesFile();
                if (coord_file !=null){
                    java.util.List list = getCoordinatesList(coord_file);
                    if (list != null) map.setCoordinates(list);
                    else return;
                }
            }
            
            //To set new coordinates from Coordinates class
            if(getValue(this.NAME).equals("new coord")){
                Coordinates set_coord = new Coordinates(map.getSuperFrame(),
                        map, true);
                set_coord.setVisible(true);
            }
            
            //To close maps settings when it's done
            if(getValue(this.NAME).equals("ok")){
                map.disposeGraphic(false);
                dispose();
            }
        }
        
        /**
         * Action method called when jFormatedTextField3 value had changed
         * @param e The action event: changing of jFormatedTextField3's value
         */
        public void propertyChange(PropertyChangeEvent e){
            
            //The interval entered in jTextField3
            int interval = ((Number)jFormatedTextField3.getValue()).intValue();
            
            //Call the autoset interval fonction of map to test the value of
            //interval and compute the number of charts
            map.autoSetInterval(interval);
            
            //Display the chart number in the 4th text field and interval in the third
            jTextField4.setText(""+map.getChartNumber());
            jFormatedTextField3.setValue(new Integer(map.getInterval()));
        }
        
        
        /**
         * Associated to the jButton3, it popups a file chooser in order the user choose a
         * .coo file
         * @return A File chosen by the user
         */
        private File getCoordinatesFile(){
            
            //The file chooser to choose a .coo file
            JFileChooser file_chooser = new JFileChooser();
            
            //The file filter to give only .coo files
            ExampleFileFilter filter = new ExampleFileFilter("coo",
                    "TESD coordinates files");
            file_chooser.setFileFilter(filter);
            
            int return_value = file_chooser.showDialog(map.getSuperFrame(),
                    null);
            
            //Returns of the choosen file or nothing
            if (return_value == JFileChooser.APPROVE_OPTION) {
                return file_chooser.getSelectedFile();
            } else  return null;
        }
        
        
        /**
         * Reads the coordinates from the chosen File and put them in a List
         * @param choosed_file The File chosen by the user
         * @return A List of coordinates of the same structure than in Maps. Will be transmited to
         * setCoordinates method of map.
         */
        private java.util.List getCoordinatesList(File choosed_file){
            
            int population_number = map.getTableWidth()-1;
            
            try{
                
                //The buffer that will read the file containing coordinates
                BufferedReader buffer = new BufferedReader(new FileReader(choosed_file));
                FileReader fr = new FileReader(choosed_file);
                
                int line_number=0;
                //A mark (a flag) for the fileReader is given
                buffer.readLine();
                //10000 is the number of characters that can be read after the mark!..
                buffer.mark(10000);
                
                //Calcul of the number of lines (i.e. popualtions)
                while (buffer.readLine()!=null){
                    line_number++;
                }
                
                //If the number of populations is not accordance between the data
                //file and the coordinate file an error is signaled
                if (line_number != population_number){
                    String error = "The number of populations in the coordinates" +
                            "file is not egal to the simul one";
                    JOptionPane.showMessageDialog(map.getSuperFrame(), error,
                            "", JOptionPane.ERROR_MESSAGE);
                    return null;
                } else {
                    //The reader come back to the mark done a the second line
                    buffer.reset();
                    
                    //Coordinates from the file are put in the all_coordiantes
                    //list of the same format than in Maps
                    StringTokenizer tokenizer;
                    java.util.List all_coordinates = new ArrayList(population_number-2);
                    for (int j=0; j < population_number; j++){
                        String line = buffer.readLine();
                        tokenizer = new StringTokenizer(line);
                        tokenizer.nextToken();
                        
                        all_coordinates.add(new Double[]{new Double(tokenizer.nextToken()), new Double(tokenizer.nextToken())});
                    }
                    return all_coordinates;
                }
                
            }catch(Exception e){
                
            }
            return null;
        }
    }
}