/*************************************************************************
 *         Copyright notice for JFreeChart and JCommon libraries          *
 *                                                                        *
 *The present application links to JFreeChart and JCommon libraries as    *
 *external jar files that are under the GNU Lesser General Public License.*
 *You can find a copy of that license at the root folder containing       *
 *the main class of TESD. More informations such as complete source codes *
 *to modify the libraries are available at this web site: www.jfree.org.  *
 *************************************************************************/

/*
 * MapsAnim.java
 *
 * Created on 8 aot 2005, 18:50
 *
 */

package projet_these.ig;

import java.awt.BorderLayout;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JSeparator;
import javax.swing.JSlider;
import javax.swing.JToolBar;
import javax.swing.JButton;
import javax.swing.JFormattedTextField;
import javax.swing.SwingConstants;
import java.text.NumberFormat;
import javax.swing.AbstractAction;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionEvent;
import org.jfree.chart.*;
import org.jfree.data.contour.*;
import java.util.*;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.annotations.XYTextAnnotation;
import org.jfree.data.Range;
import java.awt.Dimension;
import java.awt.Graphics;

/**
 * An animated version of Maps to see as spheres the popualtions
 * with a diameter function to the number or variance of TE copies
 * numbers
 * @author Yan
 */
public class MapsAnim extends Graphiques implements Runnable{
    
    private double x_max;
    /**
     * The maximum ordinate among popualtions'ones. Used to scale the plot area
     */
    private double y_max;
    /**
     * The minimum absciss among popualtions'ones. Used to scale the plot area
     */
    private double x_min;
    /**
     * The minimum ordinate among popualtions'ones. Used to scale the plot area
     */
    private double y_min;
    
    /**
     * The table containing the ordinates of the popualtions
     */
    protected  Double[] xData;
    /**
     * The table containing the ordinates of the popualtions
     */
    protected  Double[] yData;
    /**
     * A table containg the mean or variances values at each intervals of all populations
     */
    protected  Double[] zData;
    
    /**
     * The current map panel during the animation
     */
    protected ChartPanel current_map_panel;
    
    /**
     * The toolbar containing the button to control the animation
     */
    protected JToolBar toolbar;
    
    /**
     * The width of the charts
     */
    private static int chart_width = 487;
    /**
     * The height of the charts
     */
    private static int chart_height = 440;
    
    /**
     * The thread of that animation
     */
    public  volatile Thread anim;
    
    private JSlider slider;
    
    private JButton play;
    
    private JButton pause;
    
    private JButton stop;
    
    private int generation = 1;
    
    private MapsAnim mapAnim;
    
    
    /**
     * Creates a new instance of MapsAnim
     * @param super_frame The main frame of TESD
     * @param output_file_name String The file containing data (means of variances of TE 
     * copies number) to be represented as maps
     */
    public MapsAnim(MainFrame super_frame, String output_file_name) {
        
        super(super_frame, "Maps", true, output_file_name);
        
        mapAnim = this;
        
        if (this.dispose == false){
            this.getContentPane().setLayout(new BorderLayout());
            this.createToolBar();
            this.autoSetCoordinates();
            this.getContentPane().add(this.createCurrentMapChart(generation), BorderLayout.CENTER, 1);
            this.setSize(500, 450);
        }
        
    }
    
    /**
     * Create the thread
     */
    public void start(){
        if (anim == null){
            anim = new Thread(this);
            anim.start();
        }
    }
    
    /**
     * Stop the animation
     */
    public void stop(){
        anim = null;
    }
    
    /**
     * Start the animation
     */
    public void run(){
        while (generation < this.table_heigth-1 && anim != null){
            try{
                anim.sleep(0, 1);
            }catch(InterruptedException e){
            }
            changeMap(generation);
            generation ++;
        }
    }
    
    /**
     * Change the current maps panel to make the animation
     * @param generation int The current number of generation to show
     */
    public void changeMap(int generation){
        this.getContentPane().remove(1);
        this.getContentPane().add(this.createCurrentMapChart(generation), BorderLayout.CENTER, 1);
        slider.setValue(generation);
        validate();
    }
    
    
    /**
     * Create the map ChartPanel
     * @param generation int the current generation from which a map is done
     * @return ChartPanel The created ChartPanel for this generation
     */
    public ChartPanel createCurrentMapChart(int generation){
        
        ChartPanel current_map_panel = new ChartPanel(createCurrentMap(generation));
        return current_map_panel;
    }
    
    /**
     * Create the map Chart
     * @param generation int the current generation from which a map is done
     * @return JFreeChart The created MapChart for this generation to createCurrentMapChart()
     */
    public JFreeChart createCurrentMap(int generation){
        
        //Set the title of the current map: "File name(without path) - (current)Time"
        String title = output_file_name.substring(output_file_name.lastIndexOf
                (System.getProperty("file.separator"))+1) + " - Time " + generation;
        
        JFreeChart current_map = ChartFactory.createBubbleChart(title, "X", "Y",
                createCurrentMapDataset(generation),
                PlotOrientation.VERTICAL, false, true, false);
        
        //Adds the name of population on first chart
        setChartLabels(current_map);
        
        //Calls of the function that custom axis range
        setChartAxis(current_map);
        
        return current_map;
    }
    
    /**
     * Creates the dataset to plot the curretn MapChart
     * @param generation int the actual generation to plot
     * @return DefaultContourDataset the dataset to plot the MapChart
     */
    public DefaultContourDataset createCurrentMapDataset(int generation){
        
        this.setzData(generation);
        DefaultContourDataset dataset = new DefaultContourDataset("test", xData, yData, zData);
        return dataset;
    }
    
    /**
     * Set the dataset telling the size of spheres of the maps
     * @param time int the current generation
     */
    protected void setzData(int time){
        zData = new Double[getTableWidth()-1];
        //The factor by which zDatas have to be decreased in order bubbules don't overlap
        float factor = 2 / getTableMax();
        for (int i=1; i<getTableWidth(); i++){
            zData[i-1] = new Double(output_table[time][i] * factor);
        }
    }
    
    /**
     * Set random coordinates as a List containing from an array of positions computed
     * with the square root of the popuation number and sent it to the setCoordinates method
     */
    protected void autoSetCoordinates(){
        
        Integer pop_number = new Integer(getTableWidth()-1);
        
        //The square root of pop_number rounded to the upper double
        double square = java.lang.Math.ceil(java.lang.Math.sqrt(pop_number.doubleValue()));
        
        //The total number of positions in the array
        Double positions_number = new Double(Math.pow((square), 2d));
        
        //The list of map that will contain all the coordinates populations can take
        java.util.List all_coordinates = new ArrayList(positions_number.intValue());
        
        //The initial coordinates
        double x=1d, y=1d;
        
        //Filling of the list of all the possible coordinates:
        //at each columns
        for (int i=0; i<square; i++){
            
            //at each lines the positions are calculated
            for(int j=0; j<square; j++){
                all_coordinates.add(new Double[]{new Double(x), new Double(y)});
                x +=1d;
            }
            x=1d;
            y +=1d;
        }
        
        //Mixing of the coordinates
        java.util.Collections.shuffle(all_coordinates);
        
        //The x & y_Data tables will be filled from all_coordinates
        this.setCoordinates(all_coordinates);
    }
    
    /**
     * Fills the xData and yData tables from the List
     * @param all_coordinates A List containing the coordinates as tables of size 2 containing the x and y
     * coordinates of each popualtions.
     */
    protected void setCoordinates(java.util.List all_coordinates){
        
        //The maximum & minimum coodrinates used to scale the plot area
        this.x_max = 0d;
        this.y_max = 0d;
        this.x_min = 0d;
        this.y_min = 0d;
        
        //The tables containg absis and ordinates
        this.xData = new Double[getTableWidth()-1];
        this.yData = new Double[getTableWidth()-1];
        
        
        for (int i=0; i<getTableWidth()-1; i++){
            //Filling of the tables from the list
            xData[i] = ((Double[])all_coordinates.get(i))[0];
            yData[i] = ((Double[])all_coordinates.get(i))[1];
            
            //Determination of the max or min coordinates
            x_max = (((Double[])all_coordinates.get(i))[0].doubleValue() > x_max) ?
                ((Double[])all_coordinates.get(i))[0].doubleValue() : x_max;
            y_max = (((Double[])all_coordinates.get(i))[1].doubleValue() > y_max) ?
                ((Double[])all_coordinates.get(i))[1].doubleValue() : y_max;
            x_min = (((Double[])all_coordinates.get(i))[0].doubleValue() < x_min) ?
                ((Double[])all_coordinates.get(i))[0].doubleValue() : x_min;
            y_min = (((Double[])all_coordinates.get(i))[1].doubleValue() < y_min) ?
                ((Double[])all_coordinates.get(i))[1].doubleValue() : y_min;
        }
        
        //The max values are increased in order to present a spaced map
        this.x_max += 1;
        this.y_max += 1;
        
        //Min values are also reduced on the conditions they'return not to small
        if (this.x_min >=2 && this.y_min >= 2){
            this.x_min -= 1;
            this.y_min -= 1;
        }
        
    }
    
    /**
     * Add the "Pop X" labels of each populations upper on the right of the center of
     * population's circles
     * @param map The ChartPanel from which a the XYPlot will recieve the label
     */
    protected void setChartLabels(JFreeChart map){
        
        //for each population
        for (int i=0; i<getTableWidth()-1; i++){
            //"Pop x" is added at the upper right corner as XYTextAnnotation
            double label_inset = 0.25;
            XYTextAnnotation pointer = new XYTextAnnotation(
                    "Pop" + (i+1), xData[i].doubleValue() +label_inset, yData[i].doubleValue() + label_inset);
            map.getXYPlot().addAnnotation(pointer);
        }
    }
    
    /**
     * Set the rescaling of the axis when an unzoom is done
     * @param map The ChartPanel from which a the XYPlot will be properly rescalled
     */
    protected void setChartAxis(JFreeChart map){
        
        //Domain axis for VERTICAL PlotOrientation is the absis axis so Xaxis
        //Setted to the max Value of xData
        map.getXYPlot().getDomainAxis().setRange(new Range(0d, x_max),true,true);
        map.getXYPlot().getRangeAxis().setRange(new Range(0d, y_max),true,true);
    }
    
    /**
     * Create the toolbar containing the button to control the animation
     */
    protected void createToolBar(){
        
        this.toolbar = new JToolBar("Animation Control");
        toolbar.setFloatable(false);
        
        this.play = new JButton("Play");
        play.setAction(new ActionManager("play"));
        
        this.pause = new JButton("Pause");
        pause.setAction(new ActionManager("pause"));
        
        this.stop = new JButton("Stop");
        stop.setAction(new ActionManager("stop"));
        
        this.slider = new JSlider(JSlider.HORIZONTAL, 0, this.table_heigth - 1, generation);
        slider.setMajorTickSpacing((this.table_heigth - 1)/3);
        slider.setMinorTickSpacing(1);
        slider.setPaintTicks(false);
        slider.setPaintLabels(false);
        slider.addChangeListener(new ActionManager(""));
        
        toolbar.add(play);
        toolbar.add(pause);
        toolbar.add(stop);
        toolbar.addSeparator();
        toolbar.add(slider);
        getContentPane().add(toolbar, BorderLayout.SOUTH, 0);
    }
    
    private class ActionManager extends AbstractAction implements ChangeListener{
        
        private ActionManager(String action_name) {
            //A key name method to determine the action to do
            putValue(this.NAME, action_name);
        }
        
        public void actionPerformed(ActionEvent e){
            if (getValue(this.NAME).equals("play")){
                mapAnim.play.setEnabled(false);
                mapAnim.pause.setEnabled(true);
                mapAnim.stop.setEnabled(true);
                mapAnim.start();
            }
            if (getValue(this.NAME).equals("pause")){
                mapAnim.play.setEnabled(true);
                mapAnim.stop.setEnabled(true);
                mapAnim.pause.setEnabled(false);
                mapAnim.stop();
            }
            if (getValue(this.NAME).equals("stop")){
                mapAnim.play.setEnabled(true);
                mapAnim.pause.setEnabled(false);
                mapAnim.stop.setEnabled(false);
                mapAnim.stop();
                mapAnim.generation = 1;
                mapAnim.changeMap(generation);
            }
        }
        
        public void stateChanged(ChangeEvent e) {
            if (!slider.getValueIsAdjusting()) {
                if (play.isEnabled()){
                    mapAnim.stop();
                    mapAnim.generation = slider.getValue();
                    mapAnim.changeMap(generation);
                    mapAnim.play.setEnabled(true);
                    mapAnim.stop.setEnabled(true);
                    mapAnim.pause.setEnabled(false);
                } else slider.setValue(generation);
            }
        }
        
    }
}
