package projet_these.ig;

import projet_these.et.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import java.beans.*;




/**
 * Extension of a general internal frame used
 * to ask the number of migrants for homogeneous case
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 01/03/02
 */
public class MigrationMatrix1 extends SimpleQuestion {
    
    
    /**
     * Constructor of initialization of MigrationMatrix1
     * @param super_frame The main frame
     */
    public MigrationMatrix1(MainFrame super_frame) {
        
        // SimpleQuestion constructor call
        super(super_frame, "Enter the number of migrants between" + " two populations :");
        
        // Title & positionning
        setTitle("Migration Entry");
        
        // Answer suggestion
        if (getsuperFrame().model.tx_migration[0].length > 1)
            T1.setText("" + getsuperFrame().model.tx_migration[1][0]);
            T1.setToolTipText("Enter a float value (i.e. 0.5 means " +
                    "1 migration over 2 generations)");
        
        // Graphical component centering
        allCentered((JComponent) L1, (JComponent) T1, (JComponent) panel1);
        
        // Closing managment by Ok activation
        B1.addActionListener(new ok_AL());
        
        // Closing managment by Return activation
        B2.addActionListener(new return_AL());
    }
    
    
    /**
     * Access method to current instance of MigrationMatrix1
     * @return MigrationMatrix1 a reference to the current object
     */
    private MigrationMatrix1 getMigMat1() {
        return this;
    }
    
    
    /**
     * Implementation of the ActionListeneter interface
     * to listen to the events of the components
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 01/03/02
     */
    private class ok_AL implements ActionListener {
        /**
         * String for error message
         */
        String message = "";
        
        /**
         * Procedure setting the current model migration
         * matrix
         * @param  e   a click on the OK button
         */
        public void actionPerformed(ActionEvent e) {
            
            // If answer is absent from allowed answers, error signaling
            if (util_IG.errorManager(getMigMat1(), T1.getText(),
                    util_IG.POS_FLOAT0)) {
                return;
            }
            
            // Whereas migrants number vs population size compatibility is tested
            else {
                for (int i = 0; i < getsuperFrame().model.nb_pop; i++) {
                    if (Float.valueOf(T1.getText()).floatValue() *
                            (getsuperFrame().model.nb_pop - 1) >
                            getsuperFrame().model.N[i]) {
                        message =
                                "Error of data entry\n" + "The number of emigrants must be\n" +
                                "lower than the population of\n" +
                                "origin size" + Integer.toString(i + 1);
                        JOptionPane.showMessageDialog(getsuperFrame(), message, "", JOptionPane.ERROR_MESSAGE);
                        return;
                    }
                }
            }
            
            // Migration matrix creation
            getsuperFrame(). model.tx_migration =
                    new float[getsuperFrame().model.nb_pop]
                    [getsuperFrame().model.nb_pop];
            getsuperFrame().model.tx_migration[1][0] =
                    Float.valueOf(T1.getText()).floatValue();
            for (int i = 0; i < getsuperFrame().model.nb_pop; i++)
                getsuperFrame().model.tx_migration[i][i] = (float)
                getsuperFrame().model.N[i];
            for (int i = 0; i < getsuperFrame().model.nb_pop; i++) {
                for (int j = 0; j < i; j++) {
                    getsuperFrame().model.tx_migration[i][j] = (i != j) ?
                        getsuperFrame().model.tx_migration[1][0] :
                        getsuperFrame().model.N[i];
                    getsuperFrame().model.tx_migration[j][i] =
                            getsuperFrame().model.tx_migration[i][j];
                    getsuperFrame().model.tx_migration[i][i] -=
                            getsuperFrame().model.tx_migration[i][j];
                    getsuperFrame().model.tx_migration[j][j] -=
                            getsuperFrame().model.tx_migration[j][i];
                }
            }
            
            // Inner migration matrix & neigbourhood matrix are created
            getsuperFrame().model.imigration =
                    new int[getsuperFrame().model.nb_pop]
                    [getsuperFrame().model.nb_pop - 1];
            getsuperFrame().model.voisinage =
                    new Population[getsuperFrame(). model.nb_pop]
                    [getsuperFrame().model.nb_pop - 1];
            getsuperFrame().model.setNeightbourhood();
            
            /* Re-activation of:
            - Simulations menus
            - item3 (i.e. current model is completly specified*/
            getsuperFrame().item3.setEnabled(true);
            getsuperFrame().menu.setEnabled(true);
            
            // Input process is finished
            dispose();
            
            // Entries writing in the current model console
            getsuperFrame().setEntries();
            
            try{
                getsuperFrame().saveInputFile();
            }catch(java.io.IOException eio){
                
            }
            return;
        }
    }
    
    
    /**
     * Implementation of the ActionListeneter interface
     * to listen to the events of the components
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 01/03/02
     */
    private class return_AL implements ActionListener {
        
        
        /**
         * Procedure returning to the CopiesNumbers frame
         * @param  e   a click on the Return button
         */
        public void actionPerformed(ActionEvent e) {
            // One restart last step of initialisation process
            CopyNumbers cn =
                    new CopyNumbers(getsuperFrame());
            getsuperFrame().desktop.add(cn);
            try {
                cn.setSelected(true);
            } catch (java.beans.PropertyVetoException ve) {
            };
            
            // Migration rates input is paused
            dispose();
            return;
        }
    }
    
}


