package projet_these.ig;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Point;
import java.io.FileWriter;
import java.io.IOException;
import java.io.File;

import javax.swing.*;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import projet_these.et.IslandModel;
import projet_these.es.CopyAndPasteIslandModel;


/**
 * Titre : Simulation
 * Description : utilitaires de Simulation d'une invasion des ETs
 * Societe : UMR 5558
 * @author Gregory Deceliere
 * @version 01/03/02
 */
public class Simulation extends JInternalFrame {
    
    
    // compteur de fenetre
    private static int compteur = 0;
    
    // numero du processus
    private static int numero = 0;
    
    // Etat du processus de simulation
    private boolean arret = false;
    
    // Fenetre principale
    private MainFrame super_frame;
    
    // Processus de simulation
    private Thread th;
    
    // Fichier texte en ecriture
    private FileWriter sorties;
    
    // Barre de progression de la simulation
    private JProgressBar progressBar;
    
    // Identifiant du processus
    private JLabel info;
    
    // Modele simule
    private IslandModel modelSimule;
    
    private GraphicsDisplay graphs;
    
    
    /**
     * Constructor of initialization of Simulation
     * @param super_frame The main frame
     */
    public Simulation(MainFrame super_frame) {
        
        /* Constructeur d'une JInternalFrame sans titre, non
        redimensionnable, fermable, non maximisable, non iconifiable*/
        super("Simulation", false, true, false, false);
        
        // Recuperation d'une reference sur la fenetre principale
        setsuperFrame(super_frame);
        
        // Mise  jour des entrees avant de lancer une simulation
        super_frame.entries.upDate();
        
        // Desactivation des items de specification de modele
        getsuperFrame().item1.setEnabled(false);
        getsuperFrame().item2.setEnabled(false);
        
        /* Utilisation d'un font identique  celui de la barre de menu
        de la fenetre principale*/
        setBackground(getsuperFrame().getJMenuBar().getBackground());
        
        // Fermeture de la fenetre
        addInternalFrameListener(new InternalFrameAdapter() {
            public void internalFrameClosing(
                    InternalFrameEvent e) {
                compteur--;
                if (compteur == 0) {
                    getsuperFrame().
                            item1.setEnabled(
                            true);
                    getsuperFrame().
                            item2.setEnabled(
                            true);
                }
                dispose();
            }
        }
        );
        
        // Utilisation d'un gestionnaire de mise en page type GridLayout
        getContentPane().setLayout(new GridLayout(3, 1));
        
        // Initialisation de la barre de progression du processus
        progressBar = new JProgressBar();
        progressBar.setStringPainted(true);
        getContentPane().add(progressBar);
        
        
        // Initialisation de la barre d'identification du processus
        info = new JLabel("Load Status");
        getContentPane().add(info);
        
        // Initialisation des boutons Start et Stop dans Panel
        JPanel buttonPanel = new JPanel();
        JButton startButton = new JButton("Start");
        buttonPanel.add(startButton);
        startButton.addActionListener(new ActionListener() {
            public void actionPerformed(
                    ActionEvent e) {
                if (graphs != null) graphs.dispose();
                try {
                    debuter();
                } catch (IOException excep) {
                    excep.printStackTrace();
                }
            }
        }
        );
        JButton stopButton = new JButton("Stop");
        buttonPanel.add(stopButton);
        stopButton.addActionListener(new ActionListener() {
            public void actionPerformed(
                    ActionEvent e) {
                arreter();
            }
        }
        );
        getContentPane().add(buttonPanel);
        
        // Ajustement et positionnement de la fenetre de Simulation
        setSize(getPreferredSize());
        setLocation(10 * (1 + compteur % 10), 10 * (1 + compteur++ % 10));
        setResizable(false);
        setVisible(true);
    }
    
    
    /**
     * Procedure setting the reference to main frame
     */
    private void setsuperFrame(MainFrame sf) {
        
        super_frame = sf;
    }
    
    
    /**
     * Procedure returning a reference to main frame
     */
    private MainFrame getsuperFrame() {
        
        return super_frame;
    }
    
    
    /**
     * Procedure returning number of the process
     * @return int the number of the process
     */
    protected static int getNumero() {
        
        return numero;
    }
    
    
    /**
     * Starting procedure of a simulation
     */
    private void debuter() throws IOException {
        
        // Identifiant de la simulation
        String titre;
        
        numero++;
        
        /* Si c'est la premiere simulation du modele, une copie de ce dernier
        est realisee sur le disque*/
        if (numero == 1) {
            try {
                CopyAndPasteIslandModel.copyIslandModel(getsuperFrame().model);
            } catch (IOException excep) {
                excep.printStackTrace();
            }
            /*File file = new File(getsuperFrame().model.nomFic2 + ".simul1.mea");
            System.out.println(numero + "\t" + file.toString());*/
            /*if (file.exists()) {
                getsuperFrame().model.nomFic2 += ".prime";
            }*/
        }
        
        /* Une simulation ne peut-etre lancee que si elle n'est pas dej lancee.
         Sinon plusieurs processus pourraient etre lances simultaneements de la
         meme fenetre*/
        if (th == null) {
            
            modelSimule = CopyAndPasteIslandModel.pasteIslandModel(
                    getsuperFrame().model.modelFile);
            
            /*File file = new File(modelSimule.nomFic2 + ".simul1.mea");
            System.out.println(numero + "\t" + file.toString());*/
            /*if (file.exists()) {
                modelSimule.nomFic2 += ".prime";
            }*/
            
            modelSimule.nomFic2 = (modelSimule.nomFic2.endsWith(".para")) ?
                modelSimule.nomFic2.substring(0,
                    modelSimule.nomFic2.indexOf(".para", 0)) +
                    ".simul" + numero :
                modelSimule.nomFic2 + ".simul" + numero;
            
            titre = (modelSimule.nomFic2.endsWith(".txt")) ?
                modelSimule.nomFic1.substring(0,
                    modelSimule.nomFic1.indexOf(".txt", 0)) +
                    ".simul" + numero :
                modelSimule.nomFic1 + ".simul" + numero;
            titre = titre.substring(titre.lastIndexOf(System.getProperty("file.separator"))+1);
            titre = "Thread: " + titre;
            info.setText(titre);
            th = new procesSimul();
            th.setPriority(Thread.MIN_PRIORITY);
            arret = false;
            th.start();
        }
    }
    
    
    /**
     * Stopping procedure of a simulation
     */
    private void arreter() {
        
        arret = true;
    }
    
    
    /**
     * Procedure initializing the meters of window and process
     */
    public static void initMeters() {
        
        /* Initialisation du compteur static des fenetres
        utilisees par la classe Simulation*/
        compteur = 0;
        
        /* Initialisation du compteur static de simulations
        utilise	par	la classe Simulation*/
        numero = 0;
    }
    
    
    /**
     * Titre : procesSimul
     * Description : utilitaire de processus de Simulation
     * Societe : UMR 5558
     * @author Gregory Deceliere
     * @version 01/03/02
     */
    class procesSimul extends Thread {
        
        
        public void run() {
            
            /* Initialisation de la valeur et de l'intervalle de variation de
            la barre de progres du processus*/
            progressBar.setValue(0);
            progressBar.setMinimum(0);
            progressBar.setMaximum(modelSimule.tps);
            
            Runnable iteration = new Runnable() {
                
                public void run() {
                    int i = progressBar.getValue();
                    try {
                        modelSimule.simulation(i);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    i++;
                    progressBar.setValue(i);
                }
            };
            
            for (int t = 0; t <= modelSimule.tps; t++) {
                if (arret)
                    break;
                iteration.run();
            }
            
            graphs = new GraphicsDisplay(super_frame,false, super_frame, modelSimule.nomFic2);
            
            
            int graphX = (int)super_frame.getBounds().getCenterX()-(graphs.getWidth()/2);
            int graphY = (int)super_frame.getBounds().getCenterY()-(graphs.getHeight()/2);
            graphs.setLocation(new Point(graphX, graphY));
            
            graphs.setVisible(true);
            
            th = null;
        }
    }
    
}// Fin de Simulation


