package projet_these.ig;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;




/**
 * Class representing a panel containing a table
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 10/12/02
 */
public class myTable extends JPanel {
    /**
     * The table model
     */
    protected TableModel model;
    
    /**
     * The current table
     */
    public JTable table;
    
    
    /**
     * Constructor of initialization of myTable
     * @param datas The datas
     * @param colNames The columns names
     * @param col_rights boolean[] Rights of columns
     */
    public myTable(Vector datas, Vector colNames, boolean[] col_rights) {
        
        // The current table
        //JTable table;
        
        // The font
        Font f = new Font("SanSerif", Font.PLAIN, 24);
        setFont(f);
        
        // The layout manager
        setLayout(new BorderLayout());
        
        /* Construction of the model of filling with the vectors
        of datas and names*/
        model = new TableModel(datas, colNames, col_rights);
        
        // Creation of the table
        table = new JTable();
        table.setModel(model);
        table.createDefaultColumnsFromModel();
        
        // Add a scroller
        JScrollPane scrollpane = new JScrollPane(table);
        add(scrollpane);
    }
    
    
    /**
     * Procedure returning the table size
     * @return	The preferred size
     */
    public Dimension getPreferredSize() {
        
        return new Dimension(200, 200);
    }
    
    /**
     * Define the nonediatble cells of the table
     */
    public void uneditingRegularCells(){
        int col = this.table.getEditingColumn();
        int row = this.table.getEditingRow();
        if (col != -1 && row != -1) this.table.getCellEditor(row, col).stopCellEditing();
    }
}




/**
 * Titre : TableModel
 * Description : utilitaires de construction de Tableau
 * Societe : UMR 5558
 * @author Gregory Deceliere
 * @version 10/12/02
 */
class TableModel extends AbstractTableModel {
    
    /**
     * Datas
     */
    private Vector datas;
    
    /**
     * Columns names
     */
    private Vector columnNames;
    
    /**
     * Modifications indicator
     */
    private boolean modified;
    
    /**
     * Editable columns
     */
    private boolean[] col_edit;
    
    
    /**
     * Contructor of initialization of the table
     * @param datas Vector
     * @param columnsNames Vector
     * @param col_edit Vector
     */
    public TableModel(Vector datas, Vector columnsNames, boolean[] col_edit) {
        this.datas = datas;
        this.columnNames = columnsNames;
        this.col_edit = col_edit;
        modified = false;
    }
    
    
    /**
     * Procedure giving the lines number
     * @return the lines number
     */
    public int getRowCount() {
        return datas.size() / getColumnCount();
    }
    
    
    /**
     * Procedure giving the columns number
     * @return the columns number
     */
    public int getColumnCount() {
        return columnNames.size();
    }
    
    
    /**
     * Procedure giving the indexed column name
     * @return the column name
     * @param columnIndex int
     */
    public String getColumnName(int columnIndex) {
        
        String colName = "";
        if (columnIndex <= getColumnCount())
            colName = (String) columnNames.elementAt(columnIndex);
        return colName;
    }
    
    
    /**
     * Procedure giving the type of contents of an indexed column
     * @return the column type
     * @param columnIndex int
     */
    public Class getColumnClass(int columnIndex) {
        return String.class;
    }
    
    
    /**
     * Procedure giving the possibility of edit indexed cell
     * @return possibility of modify the cell
     * @param columnIndex int
     * @param rowIndex the row index
     */
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        
        return this.col_edit[columnIndex];
    }
    
    
    /**
     * Procedure giving the contents of a cell
     * @return the cell contents
     * @param columnIndex int
     * @param rowIndex the row index
     */
    public Object getValueAt(int rowIndex, int columnIndex) {
        return (String) datas.elementAt((rowIndex * getColumnCount()) +
                columnIndex);
    }
    
    
    /**
     * Procedure changing the contents of a cell
     * @param aValue Object
     * @param columnIndex int
     * @param rowIndex the row index
     */
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        datas.setElementAt(aValue,
                (rowIndex * getColumnCount()) + columnIndex);
        modified = true;
    }
}



